var crypto = require('crypto')
  , fs = require('fs')
  , path = require('path')
  , stitch = require('stitch')

var Handlebars = require('handlebars')

var createPackageFn = stitch.createPackage

stitch.createPackage = function(config) {
  var liloModulePath = path.join(
    __dirname,
    '..',
    '..',
    'node_modules'
  )

  if (typeof(config) === 'undefined') {
    config = { paths: [], dependencies: [] }
  }
  if (!config.paths) {
    config.paths = []
  }
  if (!config.dependencies) {
    config.dependencies = []
  }

  config.dependencies = config.dependencies.concat([
    path.join(liloModulePath, 'handlebars', 'dist', 'handlebars.runtime.js'),
  ])

  config.paths = config.paths.concat([ 
    path.join(liloModulePath, 'backbone'),
    path.join(liloModulePath, 'handlebars', 'dist'),
    path.join(liloModulePath, 'underscore'),
    path.join(__dirname, '..', 'client'),
    'app',
  ])

  config.compilers = {
    handlebars: function(module, filename) {
      var content;
      content = 'module.exports = ' + Handlebars.precompile(
        fs.readFileSync(filename, 'utf8')
      )

      return module._compile(content, filename)
    }
  }


  return createPackageFn(config)
}

stitch.Package.prototype.register = function(options) {
  var that = this
  var packagePath = '/app.js'

  if (!options.app) {
    throw new Error('Express app required')
  }

  if (options.packagePath) {
    packagepath = options.packagePath
  }

  options.app.use(function(req, res, next) {
    if (req.path.match(
      new RegExp(packagePath.replace('/', '\/') + '.*')
    ) === null) {
      return next()
    }

    that.compile(function(err, source) {
      if (err) {
        throw new Error(err.message)
      }

      req.app.locals.liloPackageHash = crypto.createHash('md5')
        .update(source).digest('hex')

      res.writeHead(200, {
        'Content-Type': 'text/javascript'
      })

      return res.end(source)
    })
  })
}
module.exports = stitch
